/*
 GameEngine1.cpp - Arduino Game Engine for a ST7565 based 12864 lcd1.
 Created by Thomas Baum, th.baum@online.de, 27.10.2015
 */

#include <util/delay.h>
#include <SPI.h>
#include "GameEngine1.h"
#include "Joypad.h"

Display lcd1 = Display();
Joypad joypad1 = Joypad();


GameEngine1::GameEngine1() {
  isCurserPaneActive = false;
  action = 0;
  
  joypad.init();
}

void GameEngine1::init(byte contrast) {
  lcd1.init(contrast);
  lcd1.clearVideoBuffer();
}

void GameEngine1::clearVideoBuffer() {
  lcd1.clearVideoBuffer();
}

void GameEngine1::drawPixel(byte x, byte y) {
  lcd1.drawPixel(x, y);
}

void GameEngine1::drawLine(byte x0, byte y0, byte x1, byte y1) {
  lcd1.drawLine(x0, y0, x1, y1);
}

void GameEngine1::drawBitmap(byte x, byte y, byte width, byte height, const byte *bitmap) {
  lcd1.drawBitmap(x, y, width, height, bitmap);
}

void GameEngine1::drawValue(byte x, byte y, byte value) {
  char buffer[10];
  sprintf(buffer, "%d", value);
  lcd1.drawString(x, y, buffer, false);
}

boolean GameEngine1::update() {
  if (((nextFrameMillis - millis()) > timePerFrame) && frameEndMicros) {
		nextFrameMillis = millis() + timePerFrame;
		frameCount++;
		frameEndMicros = 0;
		frameStartMicros = micros();
		joypad.update();
		return true;

	} else {
		if (!frameEndMicros) {
                        lcd1.show();
                        lcd1.clearVideoBuffer();

			frameEndMicros = micros();
			frameDurationMicros = frameEndMicros - frameStartMicros;
		}
		return false;
	}
}

boolean GameEngine1::isFrameCount(byte frame) {
  
  if ((frameCount % frame) == 0) {
                  
		  return true;
  } else {
                  
    return false;
  }
}

boolean GameEngine1::isButtonPressed(int button) {
  return joypad.isPressed(button);
}

void GameEngine1::beep() {
  tone(JOYPAD_SPEAKER_PIN, 900, 75);
}
