#include "GameEngine1.h"
#include "SPI.h"

extern const byte PROGMEM flappy[];
extern const byte PROGMEM flappyRIP[];
extern const byte PROGMEM flappyFont[];

#define GATE 25
#define GROUND 111//128-1-16
#define PIPE_WIDTH 16
#define PIPE_BORDER_HEIGHT 4
#define PIPE_BORDER_WIDTH 1
#define PIPE_REFLECTION 2

//Variablen initialisieren
byte gateHeight = GATE;
double time = 1;
double speed = 5;
double posY = 20;
byte score = 0;
boolean alive = false;
byte level;
byte y = 1;
long gate = random(25, 80);

GameEngine1 engine = GameEngine1();

void setup() {
  engine.init(20);                                //Kontrast einstellen
}

void loop() {
  if (engine.update()) {
    if (alive) {                                  //Flappy lebt
      controlPlayer();                            //Flappy steuern
      if (engine.isFrameCount(5)) {
        movePlayer();                             //Flappyposition ändern 
        moveScene();                              //Hintergrund ändern
        checkCollision();                         //Kollisionserkennung
      }
      drawScene();                                //Hintergrund malen
      engine.drawBitmap(posY, 40, 16, 16, flappy);//Flappy malen
    }
    else {            //Game over
      drawResult();
    }
  }
}

void checkCollision() {
  if (posY > GROUND - 16 || (y < 70 && y > 40 && posY > gate + gateHeight - 16) || (y < 70 && y > 40 && posY < gate - gateHeight)) {
    alive = false;
    engine.beep();
    drawPipe(gate + gateHeight, y, true);
    drawPipe(gate - gateHeight, y, false);
    drawGround();
    engine.drawBitmap(posY, 40, 16, 16, flappyRIP);
  }
}

void movePlayer() {
  time = time + 0.01;
  posY -= speed * time;
  speed -= 1.0 * time; //1.0
}

void moveScene() {
  if (y == 80) {
    y = 1;
    gate = random(25, 80);
    gateHeight--; //lvl
  }
  if (y == 60) {
    score++;
  }
  y++;
}

void drawScene() {
  drawPipe(gate + gateHeight, y, true);
  drawPipe(gate - gateHeight, y, false);
  drawGround();
  if (( y > 60 && y < 80) && score > 0) {
    engine.drawBitmap(16, 34, 16, 8, flappyFont + 16 * (score / 10));
    engine.drawBitmap(16, 24, 16, 8, flappyFont + 16 * (score % 10));
  }
}

void drawResult() {
  drawPipe(gate + gateHeight, y, true);
  drawPipe(gate - gateHeight, y, false);
  drawGround();
  engine.drawBitmap(posY, 40, 16, 16, flappyRIP);
  engine.drawBitmap(16, 34, 16, 8, flappyFont + 16 * (score / 10));
  engine.drawBitmap(16, 24, 16, 8, flappyFont + 16 * (score % 10));
  restart();
}

void restart() {
  delay(200);
  if (engine.joypad.isPressed(LEFT)) {
    alive = true;
    level = 0;
    score = 0;
    y = 1;
    time = 1;
    speed = 5;
    posY = 20;
    gate = random(25, 80);
    gateHeight = GATE;
  }
}

void controlPlayer() {
  if (engine.joypad.isPressed(LEFT)) {
    speed = 5;
    time = 1;
  }
}

void drawGround() {
  engine.drawLine(GROUND, 0, GROUND, Display::HEIGHT);
  engine.drawLine(GROUND + 3, 0, GROUND + 3, Display::HEIGHT);
}

void drawPipe(byte x, byte y, boolean ground) {
  byte pipeBorderX = ground ? x + PIPE_BORDER_HEIGHT : x - PIPE_BORDER_HEIGHT; //xposition Rand
  byte yPipeWidth = y - PIPE_WIDTH < 0 ? 0 : y - PIPE_WIDTH;
  engine.drawLine(x, y, x, yPipeWidth);//Röhreneingang
  engine.drawLine(pipeBorderX, y, pipeBorderX, yPipeWidth);//Röhrenrand
  engine.drawLine(x, y, ground ? x + PIPE_BORDER_HEIGHT : x - PIPE_BORDER_HEIGHT, y); //vorderes Seitenteil Röhrenrand
  engine.drawLine(x, y - PIPE_REFLECTION, ground ? x + PIPE_BORDER_HEIGHT : x - PIPE_BORDER_HEIGHT, y - PIPE_REFLECTION); //vorderes Seitenteil Röhrenrand Reflektion
  engine.drawLine(x, y - PIPE_WIDTH, ground ? x + PIPE_BORDER_HEIGHT : x - PIPE_BORDER_HEIGHT, y - PIPE_WIDTH); //hinteres Seitenteil Röhrenrand
  engine.drawLine(pipeBorderX, y - PIPE_BORDER_WIDTH, ground ? GROUND : 0, y - PIPE_BORDER_WIDTH); //Vorderseite
  engine.drawLine(pipeBorderX, y - PIPE_WIDTH + PIPE_BORDER_WIDTH, ground ? GROUND : 0, y - PIPE_WIDTH + PIPE_BORDER_WIDTH); //Hinterseite
  engine.drawLine(pipeBorderX, y - PIPE_REFLECTION - PIPE_BORDER_WIDTH, ground ? GROUND : 0, y - PIPE_REFLECTION - PIPE_BORDER_WIDTH); //Reflektionslinie
}

