/*
  GameEngine2.cpp - Arduino Game Engine for a ST7565 based 12864 lcd2.
  Created by Thomas Baum, th.baum@online.de, 27.10.2015
*/

#include <util/delay.h>
#include <SPI.h>
#include "GameEngine2.h"
#include "Joypad.h"

Display lcd2 = Display();
Joypad joypad2 = Joypad();


GameEngine2::GameEngine2() {
  tileX = 8;
  tileY = 8;
  tileWidth = 8;
  tileHeight = 8;
  isScorePaneActive = false;
  isBackgroundPaneActive = false;
  joypad2.init();
}

void GameEngine2::init(byte contrast) {
  lcd2.init(contrast);
  lcd2.clearVideoBuffer();
}

void GameEngine2::setTileArea(byte x, byte y, byte width, byte height) {
  tileX = x;
  tileY = y;
  tileWidth = width;
  tileHeight = height;
}

void GameEngine2::setBackground(const byte *bitmap) {
  scene = bitmap;
  isBackgroundPaneActive = true;
}

void GameEngine2::updateScore(word s) {
  score = s;
}

void GameEngine2::setScorePane(byte x, byte y) {
  scoreX = x;
  scoreY = y;
  isScorePaneActive = true;
}

boolean GameEngine2::update() {
  if (((nextFrameMillis - millis()) > timePerFrame) && frameEndMicros) { //if time to render a new frame is reached and the frame end has ran once
    nextFrameMillis = millis() + timePerFrame;
    frameCount++;

    frameEndMicros = 0;
    frameStartMicros = micros();

    joypad2.update();

    return true;

  } else {
    if (!frameEndMicros) { //runs once at the end of the frame
      if (isBackgroundPaneActive) {
        lcd2.drawBitmap(0, 0, 128, 64, scene);
      }
      for (int x = 0; x < layer1.count(); x++) {
        Tile t = layer1.getItem(x);
        lcd2.drawBitmap(t.x * 4 + tileX, t.y * 4 + tileY, 4, 4, tileSet + (4 * t.bitmap));
      }
      for (int x = 0; x < layer2.count(); x++) {
        Tile t = layer2.getItem(x);
        lcd2.drawBitmap(t.x * 4 + tileX, t.y * 4 + tileY, 4, 4, tileSet + (4 * t.bitmap));
      }
      for (int x = 0; x < layer3.count(); x++) {
        Tile t = layer3.getItem(x);
        lcd2.drawBitmap(t.x * 4 + tileX, t.y * 4 + tileY, 4, 4, tileSet + (4 * t.bitmap));
      }
      if (isScorePaneActive) {
        char buffer[10];
        sprintf(buffer, "%05d", score);
        lcd2.drawString(scoreX, scoreY, buffer, false);
      }
      lcd2.show();
      lcd2.clearVideoBuffer();

      frameEndMicros = micros();
      frameDurationMicros = frameEndMicros - frameStartMicros;
    }
    return false;
  }
}

void GameEngine2::test() {
  if (isBackgroundPaneActive) {
    lcd2.drawBitmap(0, 0, 128, 64, scene);
  }
  for (int x = 0; x < tileWidth; x++) {
    for (int y = 0; y < tileHeight; y++) {
      lcd2.drawBitmap(x * 4 + tileX, y * 4 + tileY, 4, 4, tileSet + (3 * 4));
    }
  }
  if (isScorePaneActive) {
    char buffer[10];
    sprintf(buffer, "%05d", score);
    lcd2.drawString(scoreX, scoreY, buffer, false);
  }
  lcd2.show();
  lcd2.clearVideoBuffer();
  while (true) {
  }
}

boolean GameEngine2::isFrameCount(byte frame) {

  if ((frameCount % frame) == 0) {

    return true;
  } else {

    return false;
  }
}

boolean GameEngine2::isButtonPressed(int button) {
  return joypad2.isPressed(button);
}

void GameEngine2::beep() {
  tone(JOYPAD_SPEAKER_PIN, 900, 75);
}

void GameEngine2::beep(int freq, int duration) {
  tone(JOYPAD_SPEAKER_PIN, freq, duration);
}



