/*
  GameEngine2.h - Arduino Game Engine for a ST7565 based 12864 LCD.
  Created by Thomas Baum, th.baum@online.de, 27.10.2015
*/

#ifndef _GameEngine2_H_
#define _GameEngine2_H_

#if ARDUINO >= 100
 #include "Arduino.h"
#else
 #include "WProgram.h"
#endif

#include "Display.h"
#include "Joypad.h"
#include "QueueArray.h"

 struct Tile {
    byte x;
    byte y;
    byte bitmap;
  };

const PROGMEM byte tileSet [] = {
  0x0F, 0x0F, 0x0F, 0x0F, 
  0x05, 0x0A, 0x05, 0x0A, 
  0x0F, 0x09, 0x09, 0x0F, 
  0x00, 0x06, 0x06, 0x00,
  0x09, 0x06, 0x06, 0x09, 
  0x06, 0x0F, 0x0F, 0x06,
};

class GameEngine2 {
  
	public:
          
		GameEngine2();
                void init(byte contrast);
                void setTileArea(byte x, byte y, byte width, byte height);
                void setBackground(const byte bitmap[]);
                void setScorePane(byte x, byte y);
                void updateScore(word score);
                boolean update();
                boolean isFrameCount(byte frame);
                boolean isButtonPressed(int button);              
                void playOK();
                void beep();
                void beep(int freq, int duration);
                void test();
                
                uint32_t frameCount;
                uint16_t frameDurationMicros;
                uint32_t frameStartMicros, frameEndMicros;
                uint8_t timePerFrame;
                uint32_t nextFrameMillis;
                Joypad joypad;
                QueueArray <Tile> layer1;
                QueueArray <Tile> layer2;
                QueueArray <Tile> layer3;
                
        private:
                boolean isScorePaneActive;
                boolean isBackgroundPaneActive;
                byte scoreX;
                byte scoreY;
                word score;
                byte tileX;
                byte tileY;
                byte tileWidth;
                byte tileHeight;             
                const byte* scene;            
                int frames;
                byte time;
                int lives;             
};

#endif
