/*
  GameEngine3.cpp - Arduino Game Engine for a ST7565 based 12864 lcd3.
  Created by Thomas Baum, th.baum@online.de, 27.10.2015
*/

#include <util/delay.h>
#include <SPI.h>
#include "GameEngine3.h"
#include "Joypad.h"

Display lcd3 = Display();
Joypad joypad3 = Joypad();

GameEngine3::GameEngine3() {
  for (int x = 0; x < 8; x++) {
    for (int y = 0; y < 4; y++) {
      videoRAM[x][y] = 255;
    }
  }
  joypad3.init();
}

void GameEngine3::init(byte contrast) {
  lcd3.init(contrast);
  lcd3.clearVideoBuffer();
}

void GameEngine3::setTile(byte x, byte y, byte bitmap) {
  if (x < 8 && y < 4) {
    videoRAM[x][y] = bitmap;
  }
}

byte GameEngine3::getTile(byte x, byte y) {
  return videoRAM[x][y];
}

void GameEngine3::scrollDown() {
  for (int y = 3; y > 0; y--) {
    for (int x = 0; x < 8; x++) {
      videoRAM[x][y] = videoRAM[x][y - 1];
    }
  }
  clearRow(0);
}

void GameEngine3::scrollLeft() {
  for (int x = 0; x < 7; x++) {
    for (int y = 0; y < 4; y++) {
      videoRAM[x][y] = videoRAM[x+1][y];
    }
  }
  clearRow(0);
}

void GameEngine3::clearRow(byte row) {
  for (int x = 0; x < 8; x++) {
    videoRAM[x][row] = 255;
  }
}

void GameEngine3::clearColumn(byte column) {
  for (int y = 0; y < 4; y++) {
    videoRAM[column][y] = 255;
  }
}

void GameEngine3::setTileSet(const byte *bitmap) {
  tileSet = bitmap;
}

void GameEngine3::updateScore(word s) {
  score = s;
}

void GameEngine3::showStartSplash(const byte bitmap[]) {
  lcd3.fillRectangle(0, 0, 127, 63);
  lcd3.invertRectangle(0, 0, 127, 63);
  lcd3.drawBitmap(0, 0, 128, 64, bitmap);
  lcd3.show();
  joypad3.update();
  while (!joypad3.isPressed(A)) {
    delay(200);
    joypad3.update();
  }
  delay(200);
}

void GameEngine3::showScoreSplash(byte x, byte y, word score, const byte bitmap[]) {
  lcd3.fillRectangle(0, 0, 127, 63);
  lcd3.invertRectangle(0, 0, 127, 63);
  lcd3.drawBitmap(0, 0, 128, 64, bitmap);
  char buffer[10];
  sprintf(buffer, "%05d", score);
  lcd3.drawString(x, y, buffer, false);
  lcd3.show();
  joypad3.update();
  while (!joypad3.isPressed(A)) {
    delay(200);
    joypad3.update();
  }
  delay(200);
}

boolean GameEngine3::update() {
  if (((nextFrameMillis - millis()) > timePerFrame) && frameEndMicros) { //if time to render a new frame is reached and the frame end has ran once
    nextFrameMillis = millis() + timePerFrame;
    frameCount++;

    frameEndMicros = 0;
    frameStartMicros = micros();

    joypad3.update();

    return true;

  } else {
    if (!frameEndMicros) { //runs once at the end of the frame

      for (int x = 0; x < 8; x++) {
        for (int y = 0; y < 4; y++) {
          if (videoRAM[x][y] != 255) {
            lcd3.drawBitmap(x * 16, y * 16, 16, 16, tileSet + (32 * videoRAM[x][y]));
          }
        }
      }
      lcd3.show();
      lcd3.clearVideoBuffer();

      frameEndMicros = micros();
      frameDurationMicros = frameEndMicros - frameStartMicros;
    }
    return false;
  }
}

void GameEngine3::test() {

}

boolean GameEngine3::isFrameCount(byte frame) {

  if ((frameCount % frame) == 0) {

    return true;
	frameCount = 0;
  } else {

    return false;
  }
}

boolean GameEngine3::isButtonPressed(int button) {
  return joypad3.isPressed(button);
}

void GameEngine3::beep() {
  tone(JOYPAD_SPEAKER_PIN, 900, 75);
}

void GameEngine3::beep(int freq, int duration) {
  tone(JOYPAD_SPEAKER_PIN, freq, duration);
}
