/*
  GameEngine3.h - Arduino Game Engine for a ST7565 based 12864 LCD.
  Created by Thomas Baum, th.baum@online.de, 27.10.2015
*/

#ifndef _GameEngine3_H_
#define _GameEngine3_H_

#if ARDUINO >= 100
 #include "Arduino.h"
#else
 #include "WProgram.h"
#endif

#include "Display.h"
#include "Joypad.h"

 struct Tile {
    byte x;
    byte y;
    byte bitmap;
  };

class GameEngine3 {
  
	public:
          
		GameEngine3();
                void init(byte contrast);
                void setTile(byte x, byte y, byte bitmap);
                byte getTile(byte x, byte y);
                void setTileSet(const byte bitmap[]);
                void scrollDown();
                void scrollLeft();
                void clearRow(byte row);
                void clearColumn(byte column);
                void showStartSplash(const byte bitmap[]);
                void showScoreSplash(byte x, byte y, word score, const byte bitmap[]);
                void updateScore(word score);
                boolean update();
                boolean isFrameCount(byte frame);
                boolean isButtonPressed(int button);              
                void playOK();
                void beep();
                void beep(int freq, int duration);
                void test();
                
                uint32_t frameCount;
                uint16_t frameDurationMicros;
                uint32_t frameStartMicros, frameEndMicros;
                uint8_t timePerFrame;
                uint32_t nextFrameMillis;
                Joypad joypad;
                byte videoRAM[8][4];
                
        private:
                boolean isScorePaneActive;
                boolean isBackgroundPaneActive;
                byte scoreX;
                byte scoreY;
                word score;
                byte tileX;
                byte tileY;
                byte tileWidth;
                byte tileHeight;             
                const byte* tileSet;            
                int frames;
                byte time;
                int lives;             
};

#endif
