/*
 GameEngine4.cpp - Arduino Game Engine for a ST7565 based 12864 LCD.
 Created by Thomas Baum, th.baum@online.de, 27.10.2015
 */

#include <util/delay.h>
#include <SPI.h>
#include "GameEngine4.h"
#include "Joypad.h"

Display lcd4 = Display();
Joypad joypad4 = Joypad();

GameEngine4::GameEngine4() {
  isCurserPaneActive = false;
  //location = cellBack;
  action = 0;
}

void GameEngine4::init(byte contrast) {
  lcd4.init(contrast);
  lcd4.clearVideoBuffer();
  joypad4.init();
}

void GameEngine4::clearVideoBuffer() {
  lcd4.clearVideoBuffer();
}

void GameEngine4::drawBitmap(byte x, byte y, byte width, byte height, const byte *bitmap) {
  lcd4.drawBitmap(x, y, width, height, bitmap);
}

void GameEngine4::setCurserPane(byte x, byte y, byte width, byte height, const byte *bitmap) {
  isCurserPaneActive = true;
  curserPaneX = x;
  curserPaneY = y;
  curserPaneWidth = width;
  curserPaneHeight = height;
  curserX = x + (width / 2);
  curserY = y + (height / 2);
  curser = bitmap;
}

void GameEngine4::setActionPane(byte x, byte y, byte width, byte height, const byte *bitmap, byte n) {
  isNavigationPaneActive = true;
  navigationPaneX = x;
  navigationPaneY = y;
  navigationPaneWidth = width;
  navigationPaneHeight = height;
  navigationBackground = bitmap;
  navigationEntries = n;
}

void GameEngine4::setItemPane(byte x, byte y, byte width, byte height) {
  isItemPaneActive = true;
  itemX = x;
  itemY = y;
  itemWidth = width;
  itemHeight = height;
}

void GameEngine4::setItem(byte i, const byte *bitmap) {
  item = i;
  itemIMG = bitmap;
  drawScene();
}

byte GameEngine4::getItem() {
  return item;
}

void GameEngine4::invertRectangle(byte x1, byte y1, byte x2, byte y2) {
  lcd4.invertRectangle(x1, y1, x2, y2);
}

void GameEngine4::drawInfo(const char *c) {
  lcd4.fillRectangle(4, 15, 124, 55);
  lcd4.invertRectangle(6, 17, 122, 53);
  lcd4.drawString(8, 30, c, true);
  lcd4.show();
  joypad4.update();
  while (!joypad4.isPressed(A)) {
    delay(200);
    joypad4.update();
  }
  delay(200);
  drawScene();
}

void GameEngine4::show() {

}

void GameEngine4::start() {
  drawScene();
}

void GameEngine4::drawItem() {
  lcd4.drawBitmap(itemX, itemY, itemWidth, itemHeight, itemIMG);
}

void GameEngine4::drawActionPane() {
  lcd4.drawBitmap(navigationPaneX, navigationPaneY, navigationPaneWidth, navigationPaneHeight, navigationBackground);
  lcd4.invertRectangle(navigationPaneX, action * 8, 39, (action + 1) * 8);
}

void GameEngine4::nextAction() {
  action = (action >= 5) ? 0 : action + 1;
  drawScene();
  delay(250);
}

byte GameEngine4::getAction() {
  return action;
}

void GameEngine4::drawScene() {
  lcd4.clearVideoBuffer();
  lcd4.drawBitmap(40, 0, 88, 64, scene);//thb
  drawCurser();
  drawActionPane();
  drawItem();
  char buffer[10];
  //sprintf(buffer, "%d%d", curserX, curserY);
  //engine.drawString(16, 54, buffer);
  lcd4.show();
}

void GameEngine4::setScene(const byte *bitmap) {
  scene = bitmap;
  drawScene();
}

void GameEngine4::setLocation(byte l, const byte *bitmap) {
  location = l;
  scene = bitmap;
  drawScene();
}

byte GameEngine4::getLocation() {
  delay(250);
  return location;
}

void GameEngine4::drawCurser() {
  if (isCurserPaneActive) {
    lcd4.drawBitmap(curserX-8, curserY-8, 16, 16, curser);
  }
}

boolean GameEngine4::isPointOfAction (byte x1, byte y1, byte x2, byte y2) {
  return ((x1 <= curserX && curserX <= x2) && (y1 <= curserY && curserY <= y2)) ? true : false;
}

boolean GameEngine4::update() {
  joypad4.update();
  if (isCurserPaneActive) {
    if (joypad4.isPressed(RIGHT)) {
      curserX = (curserX >= curserPaneX + curserPaneWidth) ? curserPaneX + curserPaneWidth : curserX +1;
      drawScene();
    }
    if (joypad4.isPressed(LEFT)) {
      curserX = (curserX == curserPaneX) ? curserPaneX : curserX -1;
      drawScene();
    }
    if (joypad4.isPressed(UP)) {
      curserY = (curserY == curserPaneY) ? curserPaneY : curserY -1;
      drawScene();
    }
    if (joypad4.isPressed(DOWN)) {
      curserY = (curserY >= curserPaneY + curserPaneHeight) ? curserPaneY + curserPaneHeight : curserY +1;
      drawScene();
    } 
  }
  return true;
}

boolean GameEngine4::isButtonPressed(int button) {
  return joypad4.isPressed(button);
}

void GameEngine4::showStartSplash(const byte bitmap[]) {
  lcd4.fillRectangle(0, 0, 127, 63);
  lcd4.invertRectangle(0, 0, 127, 63);
  lcd4.drawBitmap(0, 0, 128, 64, bitmap);
  lcd4.show();
  joypad4.update();
  while (!joypad4.isPressed(A)) {
    delay(200);
    joypad4.update();
  }
  delay(200);
}



