/*
  GameEngine4.h - Arduino Game Engine for a ST7565 based 12864 LCD.
  Created by Thomas Baum, th.baum@online.de, 27.10.2015
*/

#ifndef _GameEngine4_H_
#define _GameEngine4_H_

#if ARDUINO >= 100
 #include "Arduino.h"
#else
 #include "WProgram.h"
#endif

#include "Display.h"
#include "Joypad.h"

class GameEngine4 {
	public:

		GameEngine4();
                void init(byte contrast);
                void clearVideoBuffer(void);
				void drawBitmap(byte x, byte y, byte width, byte height, const byte bitmap[]);
                void setCurserPane(byte x, byte y, byte width, byte height, const byte bitmap[]);
                void setActionPane(byte x, byte y, byte width, byte height, const byte bitmap[], byte n);
                void setItemPane(byte x, byte y, byte width, byte height);
                void setItem(byte item, const byte bitmap[]);
                byte getItem();
                void invertRectangle(byte x1, byte y1, byte x2, byte y2);
                void drawActionPane();
                byte getAction();
                void nextAction();
                void drawItem();
                void drawInfo(const char *c);
                void show();
                void start();
                boolean isPointOfAction (byte x1, byte y1, byte x2, byte y2);
                void drawCurser();
                boolean update();
                void drawScene();
                void setScene(const byte bitmap[]);
                void setLocation(byte location, const byte bitmap[]);
                byte getLocation();
                boolean isButtonPressed(int button);
                void showStartSplash(const byte bitmap[]);

                Joypad joypad;
                
        private:
                boolean isCurserPaneActive;
                boolean isNavigationPaneActive;
                boolean isItemPaneActive;
                byte curserPaneX;
                byte curserPaneY;
                byte curserPaneWidth;
                byte curserPaneHeight;
                byte curserX;
                byte curserY;
                
                byte itemX;
                byte itemY;
                byte itemWidth;
                byte itemHeight;
                const byte* itemIMG;
                byte item;
                
                byte navigationPaneX;
                byte navigationPaneY;
                byte navigationPaneWidth;
                byte navigationPaneHeight;
                byte navigationEntries;
                
                byte location;
                byte action;
                const byte* navigationBackground;
                const byte* background;
                const byte* curser;
                const byte* scene;
};

#endif
